@@ -1,5 +1,8 @@
-#ifndef __DEBUG_H
-#define __DEBUG_H
+#ifndef DEBUG_H
+#define DEBUG_H
+
+#pragma GCC diagnostic ignored "-Wmissing-field-initializers"
+#pragma GCC diagnostic ignored "-Wunused-parameter"
 
 #ifdef DEBUG
 extern bool debug;
@@ -16,21 +19,17 @@ extern bool debug_z;
 #define PRECONDITION(x) if (x); else precondition( #x , __FILE__, __LINE__)
 #define NOTE(x)   tlog("%s:%d:%s: %s", __FILE__, __LINE__, __func__, #x )
 #define INFO(x,y) tlog("%s:%d:%s: " x, __FILE__, __LINE__, __func__, y )
-#define XDBG if (true)
+#define XDBG    true
 #define TLOG(x) tlog x
 #else
 #define PRECONDITION(x) // nothing
 #define NOTE(x)   // nothing
 #define INFO(x,y) // nothing
-#define XDBG if (false)
+#define XDBG    false
 #define TLOG(x)
 #endif
 #define CARP(x) tlog("%s:%d:%s: %s", __FILE__, __LINE__, __func__, #x )
 
-#if defined(DEBUG) && !defined(LOGEVENTS)
-#define LOGEVENTS 1
-#endif
-
 #endif
 
 // vim: set sw=4 ts=4 et:
