@@ -30,7 +30,7 @@ AboutDlg::AboutDlg(YActionListener* al):
     mstring copyright = copyhead
                       + copysymb + " 1997-2012 Marko Macek, "
                       + copysymb + " 2001 Mathias Hasselmann,  \n\t"
-                      + copysymb + " 2016-2024 Bert Gijsbers. \n";
+                      + copysymb + " 2016-2025 Bert Gijsbers. \n";
 
     Ladder* ladder = new Ladder();
     *ladder += label(version);
@@ -57,9 +57,10 @@ AboutDlg::AboutDlg(YActionListener* al):
     *table += new Row(label(_("Language:")), label(language));
 
     char text[123];
-    snprintf(text, sizeof text, "%s %s %s",
+    snprintf(text, sizeof text, "%s %s %s %s",
              YImage::renderName(), doubleBuffer ? _("DoubleBuffer") : "",
-             xapp->alpha() ? _("AlphaBlending") : "");
+             xapp->alpha() ? _("AlphaBlending") : "",
+             YIcon::supportSVG() ? "SVG" : "");
     *table += new Row(label(_("Renderer:")), label(text));
     if (rightToLeft)
         table->swapColumns();
