@@ -91,9 +91,10 @@ CtrlAltDelete::CtrlAltDelete(IApp* app, YWindow* parent)
         { _("_Hibernate"), actionHibernate, ICEWM_ACTION_HIBERNATE },
         { _("_Window list"), actionWindowList, ICEWM_ACTION_WINDOWLIST },
         { _("_Restart icewm"), actionRestart, ICEWM_ACTION_RESTARTWM },
-        { _("_About"), actionAbout, ICEWM_ACTION_ABOUT },
-        { _("Reload win_options"), actionWinOptions, ICEWM_ACTION_WINOPTIONS },
         { _("Reload ke_ys"), actionReloadKeys, ICEWM_ACTION_RELOADKEYS },
+        { _("Reload _toolbar"), actionToolbar, ICEWM_ACTION_TOOLBAR },
+        { _("Reload win_options"), actionWinOptions, ICEWM_ACTION_WINOPTIONS },
+        { _("_About"), actionAbout, ICEWM_ACTION_ABOUT },
         // { _("Clos_e"), actionClose, ICEWM_ACTION_NOP },
     };
     for (int i = 0; i < Count; ++i) {
@@ -122,6 +123,9 @@ CtrlAltDelete::CtrlAltDelete(IApp* app, YWindow* parent)
     for (int i = 0; i < Count; ++i) {
         int x = HORZ + (i % 3) * (w + MIDH);
         int y = VERT + (i / 3) * (h + MIDV);
+        if (i == 12 && Count == 13) {
+            x += w + MIDH;
+        }
         buttons[i]->setGeometry(YRect(x, y, w, h));
     }
 
@@ -188,24 +192,49 @@ bool CtrlAltDelete::handleKey(const XKeyEvent &key) {
         if ((k == XK_Down || k == XK_KP_Down) && m == 0) {
             int i = indexFocus();
             if (i >= 0) {
-                for (int k = 3; k < Count; k += 3) {
-                    if (buttons[(i + k) % Count]->isFocusTraversable()) {
-                        setFocus(buttons[(i + k) % Count]);
+                int j = i;
+                do {
+                    j += 3;
+                    if (Count % 3 == 1) {
+                        if (j == Count - 1)
+                            j = 0;
+                        else if (j == Count)
+                            j--;
+                        else if (j == Count + 2)
+                            j = 1;
+                    }
+                    if (j >= Count)
+                        j = i % 3;
+                    if (buttons[j]->isFocusTraversable()) {
+                        setFocus(buttons[j]);
                         break;
                     }
-                }
+                } while (j != i);
             }
             return true;
         }
         if ((k == XK_Up || k == XK_KP_Up) && m == 0) {
             int i = indexFocus();
             if (i >= 0) {
-                for (int k = Count - 3; 0 < k; k -= 3) {
-                    if (buttons[(i + k) % Count]->isFocusTraversable()) {
-                        setFocus(buttons[(i + k) % Count]);
-                        break;
+                int j = i;
+                do {
+                    j -= 3;
+                    if (Count % 3 == 1) {
+                        if (j == -2)
+                            j = Count - 1;
+                        else if (j == Count - 4)
+                            j++;
                     }
-                }
+                    if (j < 0) {
+                        j += Count - (Count % 3);
+                    }
+                    if (0 <= j && j < Count) {
+                        if (buttons[j]->isFocusTraversable()) {
+                            setFocus(buttons[j]);
+                            break;
+                        }
+                    }
+                } while (j != i);
             }
             return true;
         }
