@@ -17,21 +17,18 @@ public:
     virtual void actionPerformed(YAction action, unsigned int modifiers);
     virtual void configure(const YRect2& rect);
     virtual void repaint();
+    virtual void handleVisibility(const XVisibilityEvent&);
+    virtual void handleButton(const XButtonEvent &button);
 
     void activate();
     void deactivate();
+
 private:
-    YActionButton *lockButton;
-    YActionButton *suspendButton;
-    YActionButton *logoutButton;
-    YActionButton *restartButton;
-    YActionButton *cancelButton;
-    YActionButton *rebootButton;
-    YActionButton *shutdownButton;
-    YActionButton *aboutButton;
-    YActionButton *windowListButton;
+    int indexFocus();
+
     IApp *app;
-    YActionButton *addButton(const mstring& str, unsigned& maxW, unsigned& maxH);
+    enum { Count = 13, };
+    YActionButton* buttons[Count];
 };
 
 #endif
