@@ -95,7 +95,8 @@ void DesktopListItem::activate() {
 }
 
 WindowListBox::WindowListBox(YScrollView *view, YWindow *aParent):
-    YListBox(view, aParent)
+    YListBox(view, aParent),
+    fKeyPressed(0)
 {
 }
 
@@ -187,43 +188,37 @@ void WindowListBox::actionPerformed(YAction action, unsigned int modifiers) {
 }
 
 bool WindowListBox::handleKey(const XKeyEvent &key) {
-    if (key.type == KeyPress) {
-        KeySym k = keyCodeToKeySym(key.keycode);
-        int m = KEY_MODMASK(key.state);
+    const KeySym k = keyCodeToKeySym(key.keycode);
+    const int m = KEY_MODMASK(key.state);
 
-        switch (k) {
-        case XK_Escape:
+    if (key.type == KeyPress) {
+        fKeyPressed = k;
+        if (k == XK_Menu || (k == XK_F10 && m == ShiftMask)) {
+            YMenu* menu;
+            if (hasSelection()) {
+                menu = windowList->getWindowListPopup();
+                enableCommands(menu);
+            } else {
+                menu = windowList->getWindowListAllPopup();
+            }
+            menu->popup(nullptr, nullptr, nullptr, key.x_root, key.y_root,
+                        YPopupWindow::pfCanFlipVertical |
+                        YPopupWindow::pfCanFlipHorizontal |
+                        YPopupWindow::pfPopupMenu);
+            return true;
+        }
+        else if (k == XK_Delete) {
+            if (m & ShiftMask)
+                actionPerformed(actionKill, key.state);
+            else
+                actionPerformed(actionClose, key.state);
+            return true;
+        }
+    }
+    else if (key.type == KeyRelease) {
+        if (k == XK_Escape && k == fKeyPressed && m == 0) {
             windowList->handleClose();
             return true;
-        case XK_F10:
-        case XK_Menu:
-            if (k != XK_F10 || m == ShiftMask) {
-                if (hasSelection()) {
-                    YMenu* windowListPopup = windowList->getWindowListPopup();
-                    enableCommands(windowListPopup);
-                    windowListPopup->popup(nullptr, nullptr, nullptr,
-                                           key.x_root, key.y_root,
-                                           YPopupWindow::pfCanFlipVertical |
-                                           YPopupWindow::pfCanFlipHorizontal |
-                                           YPopupWindow::pfPopupMenu);
-                } else {
-                    YMenu* windowListAllPopup = windowList->getWindowListAllPopup();
-                    windowListAllPopup->popup(nullptr, nullptr, nullptr,
-                                              key.x_root, key.y_root,
-                                              YPopupWindow::pfCanFlipVertical |
-                                              YPopupWindow::pfCanFlipHorizontal |
-                                              YPopupWindow::pfPopupMenu);
-                }
-            }
-            break;
-        case XK_Delete:
-            {
-                if (m & ShiftMask)
-                    actionPerformed(actionKill, key.state);
-                else
-                    actionPerformed(actionClose, key.state);
-            }
-            break;
         }
     }
     return YListBox::handleKey(key);
@@ -406,6 +401,7 @@ WindowList::WindowList(YWindow *aParent):
     list(new WindowListBox(scroll, scroll))
 {
     addStyle(wsNoExpose);
+    setToplevel(true);
 
     if (listbackPixmap != null) {
         scroll->setBackgroundPixmap(listbackPixmap);
@@ -438,33 +434,33 @@ YMenu* WindowList::getWindowListPopup() {
 }
 
 WindowListPopup::WindowListPopup() {
-    addItem(_("_Restore"), -2, KEY_NAME(gKeyWinRestore), actionRestore);
-    addItem(_("Mi_nimize"), -2, KEY_NAME(gKeyWinMinimize), actionMinimize);
-    addItem(_("Ma_ximize"), -2, KEY_NAME(gKeyWinMaximize), actionMaximize);
-    addItem(_("Maximize_Vert"), -2, KEY_NAME(gKeyWinMaximizeVert), actionMaximizeVert);
-    addItem(_("MaximizeHori_z"), -2, KEY_NAME(gKeyWinMaximizeHoriz), actionMaximizeHoriz);
-    addItem(_("_Fullscreen"), -2, KEY_NAME(gKeyWinFullscreen), actionFullscreen);
+    addItem(_("_Restore"), -2, gKeyWinRestore.name, actionRestore);
+    addItem(_("Mi_nimize"), -2, gKeyWinMinimize.name, actionMinimize);
+    addItem(_("Ma_ximize"), -2, gKeyWinMaximize.name, actionMaximize);
+    addItem(_("Maximize_Vert"), -2, gKeyWinMaximizeVert.name, actionMaximizeVert);
+    addItem(_("MaximizeHori_z"), -2, gKeyWinMaximizeHoriz.name, actionMaximizeHoriz);
+    addItem(_("_Fullscreen"), -2, gKeyWinFullscreen.name, actionFullscreen);
     addItem(_("_Show"), -2, null, actionShow);
-    addItem(_("_Hide"), -2, KEY_NAME(gKeyWinHide), actionHide);
-    addItem(_("Roll_up"), -2, KEY_NAME(gKeyWinRollup), actionRollup);
-    addItem(_("_Raise"), -2, KEY_NAME(gKeyWinRaise), actionRaise);
-    addItem(_("_Lower"), -2, KEY_NAME(gKeyWinLower), actionLower);
+    addItem(_("_Hide"), -2, gKeyWinHide.name, actionHide);
+    addItem(_("Roll_up"), -2, gKeyWinRollup.name, actionRollup);
+    addItem(_("_Raise"), -2, gKeyWinRaise.name, actionRaise);
+    addItem(_("_Lower"), -2, gKeyWinLower.name, actionLower);
     addSubmenu(_("La_yer"), -2, layerMenu);
     addSubmenu(_("Tile"), -2, tileMenu);
     addSeparator();
     addSubmenu(_("Move _To"), -2, moveMenu);
-    addItem(_("Occupy _All"), -2, KEY_NAME(gKeyWinOccupyAll), actionOccupyAllOrCurrent);
+    addItem(_("Occupy _All"), -2, gKeyWinOccupyAll.name, actionOccupyAllOrCurrent);
     addItem(_("Tray _icon"), -2, null, actionToggleTray);
     addItem(_("R_ename title"), -2, null, actionRename);
     addSeparator();
-    addItem(_("Tile _Vertically"), -2, KEY_NAME(gKeySysTileVertical), actionTileVertical);
-    addItem(_("T_ile Horizontally"), -2, KEY_NAME(gKeySysTileHorizontal), actionTileHorizontal);
-    addItem(_("Ca_scade"), -2, KEY_NAME(gKeySysCascade), actionCascade);
-    addItem(_("_Arrange"), -2, KEY_NAME(gKeySysArrange), actionArrange);
+    addItem(_("Tile _Vertically"), -2, gKeySysTileVertical.name, actionTileVertical);
+    addItem(_("T_ile Horizontally"), -2, gKeySysTileHorizontal.name, actionTileHorizontal);
+    addItem(_("Ca_scade"), -2, gKeySysCascade.name, actionCascade);
+    addItem(_("_Arrange"), -2, gKeySysArrange.name, actionArrange);
     addSeparator();
-    addItem(_("_Minimize All"), -2, KEY_NAME(gKeySysMinimizeAll), actionMinimizeAll);
-    addItem(_("_Hide All"), -2, KEY_NAME(gKeySysHideAll), actionHideAll);
-    addItem(_("_Undo"), -2, KEY_NAME(gKeySysUndoArrange), actionUndoArrange);
+    addItem(_("_Minimize All"), -2, gKeySysMinimizeAll.name, actionMinimizeAll);
+    addItem(_("_Hide All"), -2, gKeySysHideAll.name, actionHideAll);
+    addItem(_("_Undo"), -2, gKeySysUndoArrange.name, actionUndoArrange);
     addSeparator();
 
     YMenu *closeSubmenu = new YMenu();
@@ -490,13 +486,13 @@ YMenu* WindowList::getWindowListAllPopup() {
 }
 
 WindowListAllPopup::WindowListAllPopup() {
-    addItem(_("Tile _Vertically"), -2, KEY_NAME(gKeySysTileVertical), actionTileVertical);
-    addItem(_("T_ile Horizontally"), -2, KEY_NAME(gKeySysTileHorizontal), actionTileHorizontal);
-    addItem(_("Ca_scade"), -2, KEY_NAME(gKeySysCascade), actionCascade);
-    addItem(_("_Arrange"), -2, KEY_NAME(gKeySysArrange), actionArrange);
-    addItem(_("_Minimize All"), -2, KEY_NAME(gKeySysMinimizeAll), actionMinimizeAll);
-    addItem(_("_Hide All"), -2, KEY_NAME(gKeySysHideAll), actionHideAll);
-    addItem(_("_Undo"), -2, KEY_NAME(gKeySysUndoArrange), actionUndoArrange);
+    addItem(_("Tile _Vertically"), -2, gKeySysTileVertical.name, actionTileVertical);
+    addItem(_("T_ile Horizontally"), -2, gKeySysTileHorizontal.name, actionTileHorizontal);
+    addItem(_("Ca_scade"), -2, gKeySysCascade.name, actionCascade);
+    addItem(_("_Arrange"), -2, gKeySysArrange.name, actionArrange);
+    addItem(_("_Minimize All"), -2, gKeySysMinimizeAll.name, actionMinimizeAll);
+    addItem(_("_Hide All"), -2, gKeySysHideAll.name, actionHideAll);
+    addItem(_("_Undo"), -2, gKeySysUndoArrange.name, actionUndoArrange);
 }
 
 void WindowList::setupClient() {
@@ -541,12 +537,9 @@ void WindowList::updateWorkspaces() {
 }
 
 void WindowList::handleFocus(const XFocusChangeEvent &focus) {
-    if (focus.type == FocusIn && focus.mode != NotifyUngrab) {
-        if (width() > 1 && height() > 1 && !getFrame()->isUnmapped()) {
-            list->setWindowFocus();
-        }
-    } else if (focus.type == FocusOut) {
-    }
+    YWindow::handleFocus(focus);
+    if (focused())
+        setFocus(list);
 }
 
 void WindowList::relayout() {
